/* main.c
 *
 * Copyright (c) 2016 Ghent University
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/**
 * @file
 * Filters and processors to be used in the main program body
 */

#include <sys/times.h>
#include <stdlib.h>
#include "graph_config.h"
#include "graph6io.h"
//#include "main.h"

/****************************Defines for timing********************************/

#include <time.h>

#if !defined(CLK_TCK) && !defined(_SC_CLK_TCK)
#include <unistd.h>
#endif
#if !defined(CLK_TCK) && defined(_SC_CLK_TCK)
#define CLK_TCK sysconf(_SC_CLK_TCK)
#endif
#if !defined(CLK_TCK) && defined(CLOCKS_PER_SEC)
#define CLK_TCK CLOCKS_PER_SEC
#endif
#if !defined(CLK_TCK)
#define CLK_TCK 60     /* If the CPU time stated by the program appears
to be out by a constant ratio, the most likely
explanation is that the code got to this point but
60 is the wrong guess.  Another common value is 100. */
#endif

/**
 * Reads a list of graph6 files from stdin and outputs to stdout only those that satisfy a given filter
 */
void main_filter_adj_list(int (*f)(ADJ_LIST_PTR, unsigned int *, int)) {

    struct tms TMS;

    unsigned long long int number_of_graphs_read = 0;
    unsigned long long int number_of_graphs_written = 0;

    int nv = 0;
    ADJ_LIST graph = {0};
    DEGREES degrees = {0};
    
    int i;
    for(i = 0; i <= MAXN; i++)
        frequency_rank[i] = 0;

    while (read_graph6_adj_list(stdin, graph, degrees, &nv)) {
        number_of_graphs_read++;

        //fprintf(stderr, "Processing graph %llu\n", number_of_graphs_read);
        if (f(graph, degrees, nv)) {

            if (!write_graph6_adj_list(stdout, graph, degrees, nv)) {
                fprintf(stderr, "Error: couldn't write graph! \n\n");
                exit(1);
            }

            number_of_graphs_written++;
        }

    }

    times(&TMS);
    unsigned int savetime = (unsigned int) TMS.tms_utime;

    fprintf(stderr, "Read %lld graphs\n", number_of_graphs_read);
    
    for(i = 0; i <= MAXN; i++)
        if(frequency_rank[i] > 0)
            fprintf(stderr, "Number of graphs with rank %d: %llu (%.2f%%)\n", i, frequency_rank[i],
                    (double) 100 * frequency_rank[i] / number_of_graphs_read);    

    fprintf(stderr, "Written %lld graphs in total\n", number_of_graphs_written);
    fprintf(stderr, "CPU time filter: %.1f seconds.\n", (double) savetime / (double) CLK_TCK);
}

/**
 * Process a list of graph6 files from stdin
 */
void main_process_adj_list(void (*p)(ADJ_LIST_PTR, unsigned int *, int)) {

    struct tms TMS;

    unsigned long long int number_of_graphs_read = 0;

    int nv = 0;
    ADJ_LIST graph = {0};
    DEGREES degrees = {0};

    while (read_graph6_adj_list(stdin, graph, degrees, &nv)) {
        number_of_graphs_read++;
        //fprintf(stderr, "Processing graph %llu\n", number_of_graphs_read);
        p(graph, degrees, nv);

    }

    times(&TMS);
    unsigned int savetime = (unsigned int) TMS.tms_utime;

    fprintf(stderr, "Read %lld graphs\n", number_of_graphs_read);
    fprintf(stderr, "CPU time filter: %.1f seconds.\n", (double) savetime / (double) CLK_TCK);
}
