/* nutgen.c
 *
 * Copyright (c) 2016 Ghent University
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/**
 * @file
 * Program which generates nut graphs only. Version of geng with
 * additional filters to only generate nut graphs.
 *
 * Usage example: ./nutgen -d2 -D3 -q -c 15 | wc -l
 */

#include <stdio.h>
#include "libs/pgeng.h"

#include "graph_config.h"
#include "graphutils.h"
#include "longmod.h"

static INT_ADJ_MATRIX int_matrix;

/*
 * Generate only nut graphs. Prunes graphs at level maxn-1 that are singular
 * and checks at level n whether the graph is a nut graph.
 */
static
int prune_longmod(unsigned int *g, int n, int maxn) {
    if (n == maxn) {
        //Uses the precomputed matrix inverse
        if (longmod_nut_property(g[n - 1], n - 1)) {
            return 0; // this is a nut graph
        } else {
            return 1;
        }
    } else if (n + 1 == maxn) {
        bits_to_int_adj_matrix(g, n, int_matrix);

        return ! inverse_longmod_matrix(n, int_matrix); // output if nonsingular
        //This method also saves and diagonalises the matrix!
    } else {
        return 0;
    }
}

int main(int argc, char **argv) {
    
    //Nutfilter also works > 32 vertices
/*
    if (MAXN > 32) {
        fprintf(stderr, "Currently only supports only up to 32 vertices\n");
        return 3;
    }
*/


    // first parameter that does not start with - is considered order
    int i=1;  // skip program name
    while (i < argc && argv[i][0] == '-') {
        i ++;
    }
    int order;
    char other;
    if (sscanf(argv[i]," %d %c",&order, &other) != 1) {
        fprintf (stderr, "Could not determine graph order from command line parameters\n");
        return 4;
    }

    if (order > 22) {
        fprintf (stderr, "Maximum order currently supported is 22\n");
        return 3;
        // limit 4294967290
        // order 22 maximum determinant 662671875
        // order 23 maximum determinant 4353564672
    }

    return  pgeng(argc, argv, prune_longmod, 0);
}
